/*
 *  (c) 2005,2006 Frank A. Zdarsky, disco | Distributed Computer Systems Lab
 *                                  University of Kaiserslautern, Germany
 *         All Rights Reserved.
 *
 *  This software is work in progress and is released in the hope that it will
 *  be useful to the scientific community. It is provided "as is" without
 *  express or implied warranty, including but not limited to the correctness
 *  of the code or its suitability for any particular purpose.
 *
 *  You are free to use this software for any non-commercial educational or
 *  research purpose, provided that this copyright notice is not removed or
 *  modified. For commercial uses please contact the respective author(s).
 *
 *  If you find our software useful, we would appreciate if you mentioned it
 *  in any publication arising from the use of this software or acknowledge
 *  our work otherwise. We would also like to hear of any fixes or useful
 *  extensions to this software.
 *
 */

package unikl.disco.math.functions;

/**
 * Interface for functions that can be displayed with <code>FunctionView</code>
 * and processed by the various operators.
 * 
 * @author Frank A. Zdarsky
 */
public interface Function {
	/**
	 * Returns the minimum x-coordinate for which the function is defined (mainly
	 * for drawing purposes).
	 * @return the lower bound of the function's definition interval
	 */
	public double getMinX();

	/**
	 * Returns the maximum x-coordinate for which the function is defined (mainly
	 * for drawing purposes).
	 * @return the upper bound of the function's definition interval
	 */
	public double getMaxX();

	/**
	 * Returns the function value at the given x-coordinate.
	 * @param x the x-coordinate
	 * @return the function value
	 */
	public double f(double x);
	
	/**
	 * Returns the gradient at the given x-coordinate or <code>Double.NaN</code> if it
	 * does not exist or is not implemented for this function.
	 * @param x the x-coordinate
	 * @return the gradient at the x-coordinate or <code>Double.NaN</code> if not defined
	 */
	public double getGradientAt(double x);
	
	/**
	 * Returns a string with an automatically created label for the function. May
	 * be an empty string, but not <code>null</code>.
	 * @return the label
	 */
	public String getLabel();
	
	/**
	 * Returns a string with a user-defined alias name for the function. If no alias
	 * is set, must return the function's label.
	 * @return the alias name
	 */
	public String getAlias();
	
	/**
	 * Allows the user to set an alias name for the function which overrides the
	 * automatically created label. The alias is used for displaying purposes and
	 * for the creation of labels for derived functions.
	 * @param alias the alias name
	 */
	public void setAlias(String alias);
}
